#include <stdio.h>
#define TRUE  1
#define FALSE 0
typedef struct treenode
{
	struct treenode *left;
	int data;
	struct treenode *right;
} BINTREE;

BINTREE* init_btree(int data);
BINTREE* make_node();
int insert_node(BINTREE *tree, int data);
void pre_order(BINTREE *tree);
void in_order(BINTREE *tree);
void post_order(BINTREE *tree);

BINTREE* init_btree(int data)
{
	BINTREE* root;
	root = make_node();
	root->left = NULL;
	root->data = data;
	root->right = NULL;
	return root;
}

BINTREE* make_node()
{
	BINTREE *temp;
	temp = (BINTREE *) malloc(sizeof(struct treenode));
	if (!temp) return NULL;
	return temp;
}

int insert_node(BINTREE *tree, int data)
{
	BINTREE *temp, *p;
	if (!tree) {
	  printf("Tree is not initialized...\n");
	  exit(0);
	}
	p = tree;
	while (p) {
	  if (data < p->data) {
		if (p->left == NULL) {
		  p->left = make_node();
		  p->left->left = NULL;
		  p->left->data = data;
		  p->left->right = NULL;
		  break;
		} else {
		  p = p->left;
		}
	  } else if (data > p->data) {
		if (p->right == NULL) {
		  p->right = make_node();
		  p->right->left = NULL;
		  p->right->data = data;
		  p->right->right = NULL;
		  break;
		} else {
		  p = p->right;
		}
	  } else {
		printf("%d is already exist...\n", data);
		return FALSE;
	  }
	}
	return TRUE;
}

void search_data(BINTREE *tree, int data)
{
	int found=0;
	BINTREE *p;
	p = tree;
	while (p) {
	  if (data < p->data) {
		if (p->left == NULL) {
		  found = 0;
		  break;
		} else
		  p = p->left;
	  } else if (data > p->data) {
		if (p->right == NULL) {
		  found=0;
		  break;
		} else
		  p = p->right;
	  } else {
		found = 1;
		break;
	  }
	}
	if (found)
		printf("\n%d is exist...\n", data);
	else
		printf("\n%d is not exist...\n", data);
}

void pre_order(BINTREE *tree)
{
  if (tree != NULL) {
	printf(" %d ", tree->data);
	pre_order(tree->left);
	pre_order(tree->right);
  }
}

void post_order(BINTREE *tree)
{
  if (tree != NULL) {
	post_order(tree->left);
	post_order(tree->right);
	printf(" %d ", tree->data);
  }
}

void in_order(BINTREE *tree)
{
  if (tree != NULL) {
	post_order(tree->left);
	printf(" %d ", tree->data);
	post_order(tree->right);
  }
}

int main()
{
	BINTREE *mytree, data;
	mytree = init_btree(100);
	insert_node(mytree, 50);
	insert_node(mytree, 150);
	insert_node(mytree, 200);
	insert_node(mytree, 120);
	insert_node(mytree, 450);
	insert_node(mytree, 30);
	insert_node(mytree, 80);
	printf("\nPre-order : ");
	pre_order(mytree);
	printf("\nIn-order : ");
	in_order(mytree);
	printf("\nPost-order : ");
	post_order(mytree);
	search_data(mytree, 200);
	search_data(mytree, 880);
	return 0;
}
/*
 Recursive Version
 insert_node(&mytree, 50);
 insert_node(&mytree, 150);

int insert_node(BINTREE **parent, int data)
{
	if (*parent == NULL) {
		*parent = make_node();
		if (*parent == NULL) {
		  printf("Memory not enough ...\n");
		  return FALSE;
		}
		(*parent)->left = NULL;
		(*parent)->data = data;
		(*parent)->right = NULL;
		return TRUE;
	}
	if (data < (*parent)->data) {
		insert_node( &((*parent)->left), data);
	} else if (data > (*parent)->data) {
		insert_node( &((*parent)->right), data);
	} else {
		printf("Duplicated data...\n");
	}
	return TRUE;
}
*/